@@ -1,18 +1,19 @@
 import logging
 import struct
-import numpy as np
 import warnings
-from typing import List, Dict, Optional, Any
+from typing import Any, Dict, List, Optional
+
+import numpy as np
 
 import qcodes as qc
+import qcodes.utils.validators as vals
 from qcodes import VisaInstrument
 from qcodes.data.data_set import DataSet
-from qcodes.instrument.channel import InstrumentChannel
 from qcodes.instrument.base import Instrument, Parameter
+from qcodes.instrument.channel import InstrumentChannel
 from qcodes.instrument.parameter import ArrayParameter, ParameterWithSetpoints
-import qcodes.utils.validators as vals
-from qcodes.utils.helpers import create_on_off_val_mapping
 from qcodes.measure import Measure
+from qcodes.utils.helpers import create_on_off_val_mapping
 
 log = logging.getLogger(__name__)
 
@@ -39,12 +40,13 @@ class LuaSweepParameter(ArrayParameter):
             start: Starting point of the sweep
             stop: Endpoint of the sweep
             steps: No. of sweep steps
-            mode: Type of sweep, either 'IV' (voltage sweep)
-                or 'VI' (current sweep)
+            mode: Type of sweep, either 'IV' (voltage sweep),
+                'VI' (current sweep two probe setup) or
+                'VIfourprobe' (current sweep four probe setup)
         """
 
-        if mode not in ['IV', 'VI']:
-            raise ValueError('mode must be either "VI" or "IV"')
+        if mode not in ['IV', 'VI', 'VIfourprobe']:
+            raise ValueError('mode must be either "VI", "IV" or "VIfourprobe"')
 
         self.shape = (steps,)
 
@@ -62,6 +64,13 @@ class LuaSweepParameter(ArrayParameter):
             self.label = 'voltage'
             self._short_name = 'vi_sweep'
 
+        if mode == 'VIfourprobe':
+            self.unit = 'V'
+            self.setpoint_names = ('Current',)
+            self.setpoint_units = ('A',)
+            self.label = 'voltage'
+            self._short_name = 'vi_sweep_four_probe'
+
         self.setpoints = (tuple(np.linspace(start, stop, steps)),)
 
         self.start = start
@@ -141,16 +150,18 @@ class TimeTrace(ParameterWithSetpoints):
 
         mode_map = {"current": "i", "voltage": "v"}
 
-        script = [f'{channel}.measure.count={npts}',
-                  f'oldint={channel}.measure.interval',
-                  f'{channel}.measure.interval={dt}',
-                  f'{channel}.nvbuffer1.clear()',
-                  '{}.measure.{}({}.nvbuffer1)'.format(channel, mode_map[mode], channel),
-                  f'{channel}.measure.interval=oldint',
-                  f'{channel}.measure.count=1',
-                  'format.data = format.REAL32',
-                  'format.byteorder = format.LITTLEENDIAN',
-                  f'printbuffer(1, {npts}, {channel}.nvbuffer1.readings)']
+        script = [
+            f"{channel}.measure.count={npts}",
+            f"oldint={channel}.measure.interval",
+            f"{channel}.measure.interval={dt}",
+            f"{channel}.nvbuffer1.clear()",
+            f"{channel}.measure.{mode_map[mode]}({channel}.nvbuffer1)",
+            f"{channel}.measure.interval=oldint",
+            f"{channel}.measure.count=1",
+            "format.data = format.REAL32",
+            "format.byteorder = format.LITTLEENDIAN",
+            f"printbuffer(1, {npts}, {channel}.nvbuffer1.readings)",
+        ]
 
         return self.instrument._execute_lua(script, npts)
 
@@ -427,8 +438,9 @@ class KeithleyChannel(InstrumentChannel):
             start: starting sweep value (V or A)
             stop: end sweep value (V or A)
             steps: number of steps
-            mode: What kind of sweep to make.
-                'IV' (I versus V) or 'VI' (V versus I)
+            mode: Type of sweep, either 'IV' (voltage sweep),
+                'VI' (current sweep two probe setup) or
+                'VIfourprobe' (current sweep four probe setup)
         """
         # prepare setpoints, units, name
         self.fastsweep.prepareSweep(start, stop, steps, mode)
@@ -448,8 +460,9 @@ class KeithleyChannel(InstrumentChannel):
             start: starting voltage
             stop: end voltage
             steps: number of steps
-            mode: What kind of sweep to make.
-                'IV' (I versus V) or 'VI' (V versus I)
+            mode: Type of sweep, either 'IV' (voltage sweep),
+                'VI' (current sweep two probe setup) or
+                'VIfourprobe' (current sweep four probe setup)
         """
 
         channel = self.channel
@@ -465,16 +478,25 @@ class KeithleyChannel(InstrumentChannel):
             meas = 'i'
             sour = 'v'
             func = '1'
+            sense_mode = '0'
 
         if mode == 'VI':
             meas = 'v'
             sour = 'i'
             func = '0'
+            sense_mode = '0'
+
+        if mode == 'VIfourprobe':
+            meas = 'v'
+            sour = 'i'
+            func = '0'
+            sense_mode = '1'
 
         script = [f'{channel}.measure.nplc = {nplc:.12f}',
                   f'{channel}.source.output = 1',
                   f'startX = {start:.12f}',
                   f'dX = {dV:.12f}',
+                  f'{channel}.sense = {sense_mode}',
                   f'{channel}.source.output = 1',
                   f'{channel}.source.func = {func}',
                   f'{channel}.measure.count = 1',
@@ -572,10 +594,9 @@ class Keithley_2600(VisaInstrument):
         knownmodels = ['2601B', '2602A', '2602B', '2604B', '2611B', '2612B',
                        '2614B', '2635B', '2636B']
         if model not in knownmodels:
-            kmstring = ('{}, '*(len(knownmodels)-1)).format(*knownmodels[:-1])
-            kmstring += 'and {}.'.format(knownmodels[-1])
-            raise ValueError('Unknown model. Known model are: ' +
-                             kmstring)
+            kmstring = ("{}, " * (len(knownmodels) - 1)).format(*knownmodels[:-1])
+            kmstring += f"and {knownmodels[-1]}."
+            raise ValueError("Unknown model. Known model are: " + kmstring)
 
         self.model = model
 
